#include <stdio.h>
#include <time.h>
#include "nttime.h"
#include "print.h"
#include "xattr.h"

static int print_nttime(NTTIME nttime)
{
    struct parsed_time p;
    if (nttime_parse(&p, nttime)) {
        printf("Could not parse nttime");
        return 1;
    }

    printf("%04d-%02d-%02dT%02d:%02d:%02d.%07dZ", p.year, p.month, p.day, p.hour, p.min, p.sec, p.nsec / 100);
    return 0;
}

static void print_attrib(uint32_t attrib)
{
    if (attrib & FILE_ATTRIBUTE_READONLY) {
        printf("READONLY");
        attrib &= ~FILE_ATTRIBUTE_READONLY;
    }
    if (attrib & FILE_ATTRIBUTE_DIRECTORY) {
        printf("DIRECTORY");
        attrib &= ~FILE_ATTRIBUTE_DIRECTORY;
    }
    if (attrib & FILE_ATTRIBUTE_ARCHIVE) {
        printf("ARCHIVE");
        attrib &= ~FILE_ATTRIBUTE_ARCHIVE;
    }
    if (attrib) {
        printf("0x%x", attrib);
    }
}

static void print_dosinfo4(struct xattr_DosInfo4* info4)
{
    int first = 1;

    printf("{");
    if (info4->valid_flags & XATTR_DOSINFO_ATTRIB) {
        if (!first)
            printf(",");
        printf("\"attrib\":\"");
        print_attrib(info4->attrib);
        printf("\"");
        info4->valid_flags &= ~XATTR_DOSINFO_ATTRIB;
        first = 0;
    }
    if (info4->valid_flags & XATTR_DOSINFO_CREATE_TIME) {
        if (!first)
            printf(",");
        printf("\"createTime\":\"");
        print_nttime(info4->create_time);
        printf("\"");
        info4->valid_flags &= ~XATTR_DOSINFO_CREATE_TIME;
        first = 0;
    }
    if (info4->valid_flags & XATTR_DOSINFO_ITIME) {
        if (!first)
            printf(",");
        printf("\"instantiationTime\":\"");
        print_nttime(info4->itime);
        printf("\"");
        info4->valid_flags &= ~XATTR_DOSINFO_ITIME;
        first = 0;
    }
    if (info4->valid_flags) {
        fprintf(stderr, "Unhandled valid flags 0x%x\n", info4->valid_flags);
    }
    printf("}");
}

void print_dosattrib(struct xattr_DOSATTRIB* dosattrib)
{
    switch (dosattrib->version) {
    case 4:
        print_dosinfo4(&dosattrib->info.info4);
        break;
    default:
        printf("Unhandled dosattrib version %d\n", dosattrib->version);
    }
}
